@@ -5,6 +5,11 @@
 # ---------------------------------------------------------------------------------------
 # version | date     | author   | changes
 # ---------------------------------------------------------------------------------------
+# 0.03    |11.10.2001| JSTENZEL | slight POD fix in "NAME" section;
+#         |          | JSTENZEL | hooks now receive the body array by reference;
+#         |          | JSTENZEL | now hook paramter: anchor object;
+#         |12.10.2001| JSTENZEL | documented new finish hook interface;
+#         |14.10.2001| JSTENZEL | added call();
 # 0.02    |22.07.2001| JSTENZEL | To make PerlPoint available under perl 5.005 which
 #         |          |          | is still widely used, I added a hint to include this
 #         |          |          | module under perl 5.005 (perl 5.6 loads parent modules
@@ -18,11 +23,11 @@
 
 =head1 NAME
 
-B<PerlPoint::Tags::Basic> - processes PerlPoint tag declarations
+B<PerlPoint::Tags> - processes PerlPoint tag declarations
 
 =head1 VERSION
 
-This manual describes version B<0.02>.
+This manual describes version B<0.03>.
 
 =head1 SYNOPSIS
 
@@ -109,11 +114,12 @@ optional or mandatory. If the slot is omitted this means that the tag does
 not need any options. The parser will I<not> accept a tag option part in this
 case.
 
-To sum it up, options and body of a tag can be declared as mandatory by TAGS_MANDATORY,
-optional by TAGS_OPTIONAL, or obsolete by TAGS_DISABLED.
+To sum it up, options and body of a tag can be declared as mandatory by
+C<TAGS_MANDATORY>, optional by C<TAGS_OPTIONAL>, or obsolete by
+C<TAGS_DISABLED>.
 
 If you need further checks you can hook into the parser by using the
-C<"hook"> ley:
+C<"hook"> key:
 
   %tags=(
          EMPHASIZE => {
@@ -123,7 +129,7 @@ C<"hook"> ley:
                        # perform special checks
                        hook => sub {
                                     # get parameters
-                                    my ($tagname, $options, @body)=@_;
+                                    my ($tagname, $options, $body, $anchor)=@_;
 
                                     # checks
                                     $rc=...
@@ -141,10 +147,12 @@ C<"hook"> ley:
         );
 
 An option hook function receives the tag name, a reference to a hash of
-option name / value pairs to check, and a body array. Using the option
-hash reference, I<the hook can modify the options>. The body array is
-I<a copy> of the body part of the stream. The hook therefore cannot
-modify the body part on parsers side.
+option name / value pairs to check, a body array reference and an
+anchor object. Using the option hash reference, I<the hook can modify the
+options>. The passed body array is I<a copy> of the body part of the stream.
+The hook therefore cannot modify the body part on parsers side.
+The anchor object can be used to store new anchors or query anchors already
+known, see \C<PerlPoint::Anchors> for details of this objects interface.
 
 The following return codes are defined:
 
@@ -154,6 +162,10 @@ The following return codes are defined:
 
 Parsing will be stopped successfully.
 
+=item PARSING_ERASE
+
+The parser will throw away the tag I<and all its content>.
+
 =item PARSING_ERROR
 
 A semantic error occurred. This error will be counted, but parsing will be
@@ -163,6 +175,11 @@ continued to possibly detect even more errors.
 
 A syntactic error occured. Parsing will be stopped immediately.
 
+=item PARSING_IGNORE
+
+The parser will ignore the tag, but stream the body. The result
+is similar to a source omitting the tag.
+
 =item PARSING_OK
 
 The checked object is declared to be OK, parsing will be continued.
@@ -174,6 +191,122 @@ please take into consideration that tag hooks might be called quite often.
 So, if checks have to be performed, users will be glad if they are
 performed quickly.
 
+And there is another hook interface. It might happen that several
+operations need parsing to be completed before they can start, like
+checking an referenced anchor which might be defined I<after> the
+referencing tag. To handle such situations, a subroutine can be
+declared as value of key C<finish>. The parser will invoke this
+code when parsing is done I<and> the tag was parsed successfully.
+(So if a C<hook> function returned an error code, the C<finish>
+hook will be ignored.)
+
+Here is an example (from an implementation of the basic tag \REF):
+
+  # afterburner
+  finish =>  sub
+              {
+               # declare and init variable
+               my $ok=PARSING_OK;
+
+               # take parameters
+               my ($options, $anchors)=@_;
+
+               # check link for being valid
+               unless (my $anchor=$anchors->query($options->{name}))
+                 {
+                  $ok=PARSING_FAILED,
+                  warn qq(\n\n[Error] Unknown link address "$options->{name}."\n);
+                 }
+               else
+                 {
+                  # link ok, get value (there is only one key/value pair
+                  # in the received hash)
+                  ($options->{__value__})=(values %$anchor);
+                 }
+
+               # supply status
+               $ok;
+              },
+
+
+Because several informations are no longer available after parsing,
+finish hooks have a different interface. They receive options and
+anchors like parsing hooks, but no line number and no body information.
+
+Options can be modified as well as in parsing hooks. Return codes are
+the same, but are evaluated slightly different according to the
+invokation time:
+
+=over 4
+
+=item PARSING_COMPLETED
+
+All right. This code is accepted for reasons of convenience,
+it is recommended to use C<PARSING_OK> instead.
+
+=item PARSING_ERASE
+
+The backend will ignore the tag and all its contents
+(which means its body).
+
+=item PARSING_ERROR
+
+A semantic error occurred. This error will be counted.
+
+=item PARSING_FAILED
+
+An error occured. Because parsing is already finished,
+this will be counted as an I<sematic> error.
+
+This code is accepted for reasons of convenience,
+it is recommended to use C<PARSING_ERROR> instead.
+
+=item PARSING_IGNORE
+
+The backend will ignore the tag, but process its body.
+This simply means that the tag takes no effect.
+
+=item PARSING_OK
+
+All right.
+
+=back
+
+The order of finish hook invokation can I<differ> from the order
+of tag usage. Do not depend on it.
+
+A finish hook is I<not> invoked unless the tag was processed
+and streamed successfully at parsing time. This simply means
+if the parsing hook returned C<PARSING_OK>, or if there was
+no parsing hook at all.
+
+
+=head2 Using other tag definitions
+
+One can invoke hooks of I<any other registered tag>. This is
+powerful, but dangerous. Nevertheless, it helps to emulate
+other tags, for example if an old interface (tag and option
+names) shall be preserved but the new functionality shall
+be used (without being copied between tag modules). To invoke
+a foreign hook, call \C<PerlPoint::Tags::call()> (fully
+qualified function name) with tag name, hook type and
+parameters, like so:
+
+ $rc=PerlPoint::Tags::call('TAG', 'hook', @_);
+
+Valid hook types are "hook" and "finish" (currently). If the
+tag is not registered, or has no hook of the specified type,
+an undefined value is supplied, otherwise the return code of
+the invoked function.
+
+It is not checked if you call a "hook" function from a "finish"
+hook or vice versa. Take care!
+
+This feature is made available to interchange hooks between
+several tag definition modules. If you want to share hook
+functions between tags declared by the I<same> tag module, it
+is recommended to use common Perl techniques.
+
 
 =head2 Tag activation
 
@@ -300,8 +433,7 @@ require 5.00503;
 package PerlPoint::Tags;
 
 # declare package version (as a STRING!!)
-$VERSION="0.02";
-
+$VERSION="0.03";
 
 
 # = PRAGMA SECTION =======================================================================
@@ -348,8 +480,8 @@ sub import
      if ($declared=~s/^:(.+)$/$1/)
        {
         # check if this set was declared (and declared as expected)
-        confess "Use of undeclared tag set $declared in class $class" unless exists $sets->{$declared};
-        confess "Set $declared of class $class was not declared by an array reference" unless ref($sets->{$declared}) eq 'ARRAY';
+        confess "[BUG] Use of undeclared tag set $declared in class $class" unless exists $sets->{$declared};
+        confess "[BUG] Set $declared of class $class was not declared by an array reference" unless ref($sets->{$declared}) eq 'ARRAY';
 
         # add all set tags to the list
         unshift(@_, sort @{$sets->{$declared}});
@@ -359,7 +491,7 @@ sub import
        }
 
      # ok, this is a tag - first, check if it was declared
-     confess "Use of undeclared tag $declared in class $class" unless exists $tags->{$declared};
+     confess "[BUG] Use of undeclared tag $declared in class $class" unless exists $tags->{$declared};
 
      # does the new tag overwrite an earlier declaration?
      if (exists $tagdefs->{$declared})
@@ -406,6 +538,24 @@ sub addTagSets
  }
 
 
+# call a hook function of a registered tag
+sub call
+ {
+  # get and check parameters
+  my ($tag, $type, @pars)=@_;
+  confess "[BUG] Missing tag name.\n" unless $tag;
+  confess "[BUG] Missing function type.\n" unless $type;
+  confess "[BUG] Invalid function type \"$type\".\n" unless $type=~/^(finish|hook)$/;
+  
+  # flag an error in several cases, let the caller deal with it
+  return undef unless     exists $tagdefs->{$tag}
+                      and exists $tagdefs->{$tag}{$type};
+  
+  # ok, call the function
+  &{$tagdefs->{$tag}{$type}}(@pars);
+ }
+
+
 
 # flag successful loading
 1;
